<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-irods
 * @author    Rob Campbell <rcampbel@purdue.edu>
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// No direct access

defined('_HZEXEC_') or die();

//require_once PATH_APP . DS . 'plugins' . DS . 'metadata' . DS . 'irods' . DS . 'irods.php';

/**
 * HUBzero extension of flysystem local adapter
 */
class IrodsAdapter extends \League\Flysystem\Adapter\Local
{
	/**
	* Contructor: Save mount path for later use
	*
	**/
	function __construct($path, $mount_path)
	{
		$this->mount_path = $mount_path;

		parent::__construct($path);
	}

	/**
	* Get base directory (path prefix with mount path removed)
	*
	* @return  string	base dir with trailing DS
	**/
	public function getBaseDir()
	{
		$baseDir = str_replace($this->mount_path, '', $this->pathPrefix);

		return $baseDir;
	}

	/**
	* @inheritdoc
	*/
	public function write($path, $contents, League\Flysystem\Config $config)
	{
		$path = $this->adjustName($path);

		$result = parent::write($path, $contents, $config);

		// Set metadata
		if (false != $result)
		{
			Event::trigger('metadata.captureInitialMetadata',array($this->getBaseDir() . $path));
			//plgMetadataIrods::captureInitialMetadata($this->getBaseDir() . $path);
		}

		return $result;
	}

	/**
	* @inheritdoc
	*/
	public function writeStream($path, $resource, League\Flysystem\Config  $config)
	{
		$path = $this->adjustName($path);

		$result = parent::writeStream($path, $resource, $config);

		// Set metadata
		if (false != $result)
		{
			Event::trigger('metadata.captureInitialMetadata',array($this->getBaseDir() . $path));
			//plgMetadataIrods::captureInitialMetadata($this->getBaseDir() . $path);
		}

		return $result;
	}

	/**
	* @inheritdoc
	*/
	public function listContents($directory = '', $recursive = false)
	{
		$result = [];
        $location = $this->applyPathPrefix($directory) . $this->pathSeparator;

		if (! is_dir($location))
		{
			return [];
		}

		if ('' == $location)
		{
			return [];
		}

		$cdir = scandir($location);

		foreach ($cdir as $key => $value) {
			if (!in_array($value, array(".", "..")))
			{
				$path = $location.$value;

				if ($recursive && is_dir($path)) {
					$result[$value] = $this->listContents($path,$recursive);
				}
				else {
					$file = new SplFileInfo($path);
					$result[] = $this->normalizeFileInfo($file);
				}
			}
		}

		return array_filter($result);
	}

	/**
	* @inheritdoc
	*/
	public function deleteDir($dirname, $recursing=false)
	{
		if (!$recursing) {
			$location = $this->applyPathPrefix($dirname); // .$this->pathSeparator;
		}
		else {
			$location = $dirname;
		}

		if (!is_dir($location)) {
			return false;
		}

		if ('' == $location) {
			return false;
		}

		// Recursivly delete contents
		$cdir = scandir($location);

		foreach ($cdir as $key => $value) {
			if (!in_array($value, array(".", ".."))) {

				$path = $location.$this->pathSeparator.$value;

				if (is_dir($path)) {
					if (!$this->deleteDir($path, true)) {
						return false;
					}
				}
				else {
                    unlink($path); // covers symlinks, files
				}
			}
		}

		// Delete given dir
		return rmdir($location);
	}

	/**
	* @inheritdoc
	*/
	public function rename($path, $newpath)
	{
		// TEMP FIX: If dir, silently do nothing
		$prefixed = $this->applyPathPrefix($path);

		if (is_dir($this->applyPathPrefix($path))) // $prefixed . $this->pathSeparator))
		{
			return false;
		}
		else
		{
			return parent::rename($path, $this->adjustName($newpath));
		}
	}

	/**
	 * Change filename to be valid for use with our systems: legal chars: Aa-Zz0-9.-_
	 *
	 * @param   string  $path      Original path including file name
	 * @return  string             Path with unsupported chars removed from file name
	 */
	public function adjustName($path)
	{
		// Separate containing dir's path, prep it for reattachment
		$dir  = dirname($path);
		if ($dir == '.')
		{
			$dir = '';
		}
		if ($dir != '' && substr($dir, -1) != DS)
		{
			$dir .= DS;
		}

		// Replace whitespace with underscores
		$without_whitespace = preg_replace('/\s+/', '_', basename($path));

		// Remove unsupported chars in file name
		$name = preg_replace('/[^A-Za-z0-9\.\-\_]/', '', $without_whitespace);

		return $dir . $name;
	}
}
