/**
 * @package     hubzero-irods
 * @file        app/plugins/handlers/irods/assets/js/geomap.js
 * @copyright   Copyright 2016 HUBzero Foundation, LLC.
 * @license     http://opensource.org/licenses/MIT MIT
 *
 * GABBS Map Viewer
 *
 * Display Modes
 *  - Standard:  All must use GABBS "standard" projection (see below).
 *               Layers with non-standard projection or others errors are noted and NOT displayed.
 *               Multiple base providers and base layers are selectable.
 *  - Alternate: All layers must use the same NON-standard projection.
 *               They are displayed with blank base layer.
 *               Base provider and base layers selection are NOT displayed.
 * Assume:
 *  1. All layers (raster and vector) are transformed to GABBS "standard" projection prior to push to WMS server
 *  2. Layers with transform errors are added to WMS server as-is (using native projection).
 */

function logtime(msg)
{
	try
	{
		console.log(Math.round(performance.now()),msg);
	}
	catch (e)
	{
		// Do nothing (Safari doesn't recognize "performance".)
	}
}

var geoMap =
{
	STD_PROJ_STR  : 'EPSG:4326',  // GABBS' "standard" projection (WGS 84)
	DRAW_DELAY    : 2000,         // Wait time to avoid race w/OL
	BASIC         : 'basicbase',  // Base layer providers - Correspond to ID of html element
	BING          : 'bingbase',   //  "
	GOOGLE        : 'googlebase', //  "
	OSM           : 'osmbase',    //  "
	availStyles   : [],           // List of style available for vector layers
	baseProvider  : '',           // Current (selected) base provider
	wmsServer     : '',           // WMS host (e.g GeoServer)
	ws            : '',           // Workspace
	layers        : [],           // Array of layer names
	types         : [],           // Corresponding layer types ('raster'/'vector')
	projs         : [],           // Corresponding layer projections
	nlim          : [],           // Corresponding layer bounds...
	elim          : [],           //  "
	wlim          : [],           //  "
	slim          : [],           //  "
	styles        : [],           //  "
	stdDispMode   : false,        // Display mode (std vs alt)
	erred         : [],           // Names of layers that cannot be displayed on map
	map           : {},           // OpenLayers map obj
	overlays      : {},           // holds non-base, user-specified OL2 layers
	highlights    : {},           // Layer for emphasizing selected feature
	baseProjStr   : '',           // Base layer projection (as string "EPSG:....")
	baseProjObj   : {},           //  "
	overlayProjStr: '',           // Overlay layer(s) projection (as string "EPSG:....")
	overlayProjObj: {},           //  "
	zoomLimit     : 0,            // Limit zoom out based on base provider
	panelAdjusting: false,        // TODO suppress map redraw during adjustment

	init: function(pWmsServer,pWs,pLayers,pTypes,pProjs,pErred,pNlim,pElim,pWlim,pSlim,pStyles)
	{
		logtime('start');

		//trace('start');
		this.baseProvider = this.OSM;

		// Save off common paramters to working vars
		this.wmsServer   = pWmsServer;
		this.ws          = pWs;
		this.erred       = pErred;
		this.availStyles = pStyles.split(/\s+/);

		//  Find overlay projection and display mode
		this.overlayProjStr = '?';

		// Detect standard display mode: ANY overlay uses our standard projection
		for (var i=0; i<pLayers.length; i++)
		{
			if (this.STD_PROJ_STR == pProjs[i])
			{
				this.stdDispMode    = true;
				this.overlayProjStr = this.STD_PROJ_STR; // Go with standard display mode
				break;
			}
		}

		// Display mode still undetermined? Try to detect alt mode conditions.
		if ('?' == this.overlayProjStr)
		{
			// Alternate display mode: ALL overlays use the same, non-standard projection
			for (var i=0; i<pLayers.length; i++)  // Skips if there are zero, non-errred layers
			{
				if (pProjs[i] == pProjs[0])  // Correctly handles single layers (compares elem 0 to itself)
				{
					this.stdDispMode    = false;     // Stay with alt display mode
					this.overlayProjStr = pProjs[0]; // Stay with non-std projection but keep checking!
				}
			}
		}

		// Display remains undetermined? Can't show any layers
		if ('?' == this.overlayProjStr)
		{
			this.stdDispMode    = true;      // Default to std mode to show errors
			this.overlayProjStr = this.STD_PROJ_STR;
		}

		// Examine & save valid layer data to working arrays:
		for (var i=0; i<pLayers.length; i++)
		{
			if (pProjs[i] == this.overlayProjStr)
			{
				this.layers[i] = pLayers[i];
				this.types[i]  = pTypes[i];
				this.projs[i]  = pProjs[i];
				this.nlim[i]   = pNlim[i];
				this.elim[i]   = pElim[i];
				this.wlim[i]   = pWlim[i];
				this.slim[i]   = pSlim[i];
			}
			else
			{
				this.erred[this.erred.length] = pLayers[i];
			}
		}

		// Expand display to fill window
		// From http://stackoverflow.com/questions/2872967/how-to-grow-an-iframe-depending-on-the-size-of-its-contents
		var useHeight = parent.document.documentElement.clientHeight; //IE 6+ in 'standards compliant mode'
		var useWidth  = parent.document.documentElement.clientWidth;
		if (typeof (parent.window.innerWidth) == 'number')
		{
			useHeight = parent.window.innerHeight; //Non-IE
			useWidth  = parent.window.innerWidth;
		}
		else if (parent.document.body && (parent.document.body.clientWidth || parent.document.body.clientHeight)) {
			useHeight = parent.document.body.clientHeight; //IE 4 compatible
			useWidth = parent.document.body.clientWidth;
		}
		$('#map-viewer').height(useHeight - 120);
		$('#map-viewer').width( useWidth  - 110);
		$.fancybox.update() // NOTE Assumes we're inside a fancybox

		var that = this; // JSTC

		// Make panel resizeable
		$("#panel").resizable({handles: 'e'});
		$('#panel').resize(function() {that.panelAdjustment();}); // When panel is resized, adjust map to compensate  JSTC
		$(window).resize(function()   {that.panelAdjustment();}); // Handle window resize       JSTC

		// Build map based on standard or alternate display mode
		if (this.stdDispMode)
		{
			// Support changing base layer provider
			$('#'+this.baseProvider).css("font-weight","bold"); // Starting value
			$('#'+this.BASIC ).click(function() {that.changeBaseProvderTo(that.BASIC );});  // JSTC
			$('#'+this.GOOGLE).click(function() {that.changeBaseProvderTo(that.GOOGLE);});  // JSTC
			$('#'+this.BING  ).click(function() {that.changeBaseProvderTo(that.BING  );});  // JSTC
			$('#'+this.OSM   ).click(function() {that.changeBaseProvderTo(that.OSM   );});  // JSTC

			this.buildMapStd(); // Standard display mode
		}
		else
		{
			$('#basepros').hide();
			this.buildMapAlt(); // Alternate display mode
		}

		//trace('end');
	},

	// Build map in standard display mode. Called at start up AND when base provider changes
	buildMapStd: function()
	{
		this.overlays   = [];
		this.highlights = [];

		this.createMapStd();        // Initialize OL's map object
		this.createBaseLayersStd(); // Add base layers to map - only one is visible
		this.createOverlayLayers(); // Add user-specified layer(s) to map
		this.createFeatureLayer();  // Add vector layer for higlighting selected feature(s)

		// Draw map by auto-zoom/center'ing - Race condition? Wait before zoom center
		var that = this;  // JSTC
		setTimeout( function()
			{
				// Center viewer and zoom in on (or near) user's data/image(s)
				that.zoomAndCenterStd();   // JSTC
				logtime('stop');
			}
			,this.DRAW_DELAY
		);

		this.prepGetFeature();  // Respond to mouse clicks by highlighting feature and displaying info
		this.initInfoPanel();
	},

	createMapStd: function()
	{
		this.initOpenlayers();

		// Build OL map object - w/no controls (for now)
		var maxRes       = 156543.03390625;
		var maxExtBounds = new OpenLayers.Bounds(-128*maxRes,-128*maxRes,128*maxRes,128*maxRes);
		var opts;
		try
		{
			var that  = this; // JSTC

			switch (this.baseProvider)
			{
				case this.BASIC : // Base layer on WMS server NOTE EPSG:4326
					this.baseProjStr = 'EPSG:4326';
					this.zoomLimit   = 2;
					opts  =
					{
						controls   : []
						,projection: new OpenLayers.Projection(that.baseProjStr) // JSTC
						,theme     : null
					};
					break;
				case this.GOOGLE :
					this.baseProjStr = 'EPSG:900913'; // = 'EPSG:3857'
					this.zoomLimit   = 2;
					opts  =
					{
						controls          : []
						,maxExtent        : maxExtBounds
						,maxResolution    : maxRes
						,projection       : new OpenLayers.Projection(that.aseProjStr)     // JSTC
						,displayProjection: new OpenLayers.Projection('EPSG:4326') // TODO Required?
						,theme            : null
					};
					break;
				case this.BING :
					this.baseProjStr = 'EPSG:900913';
					this.zoomLimit   = 1;
					opts  =
					{
						controls          : []
						,maxExtent        : maxExtBounds
						,maxResolution    : maxRes
						,numZoomLevels    : 19
						,projection       : new OpenLayers.Projection(that.baseProjStr)    // JSTC
						,displayProjection: new OpenLayers.Projection('EPSG:4326') // TODO Required?
						,theme            : null
					};
					break;
				case this.OSM :
					this.baseProjStr = 'EPSG:900913';
					this.zoomLimit   = 2;
					opts  =
					{
						controls          : []
						,projection       : new OpenLayers.Projection(that.baseProjStr)   // JSTC
						,displayProjection: new OpenLayers.Projection('EPSG:900913') // TODO Required?
						,theme            : null
					};
					break;
			}

			this.map = new OpenLayers.Map('map', opts); // Div must exist

			// Create these as early as possible to cause bounds transformation (for zoom/center) to work 1st time
			// TODO Still needed this early?
			this.overlayProjObj = new OpenLayers.Projection(this.overlayProjStr);
			this.baseProjObj    = new OpenLayers.Projection(this.baseProjStr   );

			this.limitZoomOut()  // Reset zoom and try to center if user goes too far out
		}
		catch (e)
		{
			//trace('createMapStd: '+e);
			this.error('Unable to create map.');
		}

		this.addControls();
		//trace('reCreateMapStd');
	},

	createBaseLayersStd: function()
	{
		// Create default base layers
		var bases = [];
		try
		{
			switch (this.baseProvider)
			{
				case this.BASIC:
					var default_base = true;
					var blank_base   = new OpenLayers.Layer(" Blank", {isBaseLayer: true});

					// Set base layer to blank if any vector layers
					for (var i=0; i<this.types.length; i++)
					{
						if (this.types[i] == 'vector')
						{
							bases[bases.length] = blank_base;
							default_base        = false;
							break;
						}
					}

					// Create other base layers
					var baseSet = [];
					baseSet.push({label:' World - Country Borders',ws:'geoshare',ln:'worldmap_world_1004'});
					baseSet.push({label:' US - State Borders'     ,ws:'topp'    ,ln:'states'             });
					for (var i=0; i<baseSet.length; i++)
					{
						var thisBaseProjStr = this.baseProjStr;
						bases[bases.length] = new OpenLayers.Layer.WMS(
							baseSet[i].label
							,this.wmsServer + "/" + baseSet[i].ws + "/wms"
							,{
								LAYERS : baseSet[i].ln
								,STYLES: 'line'
								,format: 'image/png'
							}
							,{
								singleTile  : true
								,ratio      : 1
								,isBaseLayer: true
								,yx         : {thisBaseProjStr: false} // JSTC
							}
						);
					}

					// Add blank if not already in
					if (default_base)
					{
						bases[bases.length] = blank_base;
					}
					break;

				case this.GOOGLE:
					var gNameTypes = [];
					gNameTypes.push({label:' Google Roadmap'  ,typeId:google.maps.MapTypeId.ROADMAP  });
					gNameTypes.push({label:' Google Satellite',typeId:google.maps.MapTypeId.SATELLITE});
					gNameTypes.push({label:' Google Terrain'  ,typeId:google.maps.MapTypeId.TERRAIN  });
					gNameTypes.push({label:' Google Hybrid'   ,typeId:google.maps.MapTypeId.HYBRID   });
					for (var i=0; i<gNameTypes.length; i++)
					{
						bases[bases.length] = new OpenLayers.Layer.Google(gNameTypes[i].label,
						{
							type        : gNameTypes[i].typeId
							,isBaseLayer: true
						});
					}
					break;

				case this.BING:
					var bingTypes = ['AerialWithLabels','Road','Aerial'];
					for (i=0; i<bingTypes.length; i++)
					{
						bases[bases.length] = new OpenLayers.Layer.Bing(
						{
							type        : bingTypes[i]
							,name       : ' Bing Maps '+bingTypes[i]
							,key        : 'AhJbCY6RNyaw4P5EkEq4xgI75lO8rlIyO55HWJg9Va1r5axbZLNywWQCvLPC9Ucy' // 'AqTGBsziZHIJYYxgivLBf0hVdrAk9mWO5cQcb8Yux8sW5M8c8opEC2lZqKR1ZZXf'
							,isBaseLayer: true
						})
					}
					break;

				case this.OSM:
					bases[bases.length] = new OpenLayers.Layer.OSM(' OpenStreetMap Default');
					break;
			}

			this.map.addLayers(bases);
		}
		catch (e)
		{
			//trace('createBaseLayers: '+e);
			this.error('Unable to create base layer(s).');
		}
		//trace('createBaseLayers');
	},

	// Use WMS layers for all of user's raster and vector data/files
	createOverlayLayers: function()
	{
		var styleInc = 0; // Keep track of the next vector style to use
		var baseProj = this.BaseProjStr;
		var that     = this;

		try
		{
			for (var i=0; i<this.layers.length; i++)
			{
				// Set style: raster uses default, vector changes
				this.styles[i] = '';
				if (this.types[i] == 'vector')
				{
					this.styles[i] = this.availStyles[styleInc++ % this.availStyles.length];
				}

				// Create layer
				this.overlays[i] = new OpenLayers.Layer.WMS(
					' '+this.layers[i]
					,this.wmsServer + "/" + this.ws + "/wms"
					,{
						LAYERS      : that.layers[i]    // JSTC
						,STYLES     : that.styles[i]    // JSTC
						,transparent: true
						,format     : 'image/png'
					}
					,{
						singleTile  : true
						,ratio      : 1
						,isBaseLayer: false
						,yx         : {baseProj: false} // JSTC
						,opacity    : 0.6
					}
				);
			}

			this.map.addLayers(this.overlays); // Add user's (non-base) layers to map
		}
		catch (e)
		{
			console.error('ERROR in createOverlayLayers: '+e);
			this.error('Unable to create overlay layer(s).');
		}
		//trace('createOverlayLayers');
	},

	// Zoom in & center view on user's map data area
	zoomAndCenterStd: function()
	{
		// Find bounds that include areas from all layers
		var maxBounds = new OpenLayers.Bounds.fromArray(this.limitBounds(this.overlayProjObj,this.wlim[0],this.slim[0],this.elim[0],this.nlim[0]));
		maxBounds = maxBounds.transform(this.overlayProjObj,this.baseProjObj);

		for (var i=1; i<this.nlim.length; i++)
		{
			var newBounds = new OpenLayers.Bounds.fromArray(this.limitBounds(this.overlayProjObj,this.wlim[i],this.slim[i],this.elim[i],this.nlim[i]));
			newBounds = newBounds.transform(this.overlayProjObj,this.baseProjObj);

			if (newBounds.bottom < maxBounds.bottom) maxBounds.bottom = newBounds.bottom;
			if (newBounds.top    > maxBounds.top   ) maxBounds.top    = newBounds.top   ;
			if (newBounds.left   < maxBounds.left  ) maxBounds.left   = newBounds.left  ;
			if (newBounds.right  > maxBounds.right ) maxBounds.right  = newBounds.right ;
		}

		// Adjust view
		var center = maxBounds.getCenterLonLat();
		var zoom   = this.map.getZoomForExtent(maxBounds);
		this.map.setCenter(center,zoom);

		//trace('zoomcenter');
	},

	buildMapAlt: function()
	{
		this.overlays   = [];
		this.highlights = [];

		this.initOpenlayers();
		try
		{
			this.baseProjStr = this.overlayProjStr;
			this.baseProjObj = new OpenLayers.Projection(this.baseProjStr);
			this.zoomLimit   = 2;

			// Create map
			var bounds = new OpenLayers.Bounds.fromArray(this.limitBounds(this.baseProjObj,this.wlim[0],this.slim[0],this.elim[0],this.nlim[0]));
			var that   = this; // JSTC
			var opts   =
			{
				controls   : []
				,maxExtent : bounds
				,projection: that.baseProjObj // JSTC
				,theme     : null
			};
			this.map = new OpenLayers.Map('map', opts); // Div must exist

			this.limitZoomOut();
			this.addControls();

			// Create base layer
			var blank_base = new OpenLayers.Layer(" Blank", {isBaseLayer: true});
			this.map.addLayer(blank_base);

			this.createOverlayLayers(); // Add user's data layer(s)
			this.createFeatureLayer();  // Add vector layer for higlighting selected feature(s)

			// Draw map by zooming NOTE Race condition? Wait before zooming
			var thisMap = this.map;
			setTimeout( function()
				{
					thisMap.zoomToExtent(bounds); // JSTC
					logtime('stop');
				}
				,this.DRAW_DELAY
			);

			this.prepGetFeature(); // Respond to mouse clicks by highlighting feature and displaying info
			this.initInfoPanel();
		}
		catch (e)
		{
			//trace('buildMapAlt: '+e);
			this.error('Unable to create map.');
		}
	},

	initOpenlayers: function()
	{
		// Initialize OpenLayer settings
		OpenLayers.IMAGE_RELOAD_ATTEMPTS = 5;            // "Pink tile avoidance" (?)
		OpenLayers.DOTS_PER_INCH         = 25.4 / 0.28;  // "Make OL compute scale according to WMS spec" (?)
		OpenLayers.ProxyHost             = "/irods/?task=geosrvproxy&url="; // Our proxy for OL to hit GS - only for some calls!
	},

	limitZoomOut: function()
	{
		// Limit zoom-out: reset zoom and try to center if user goes too far out
		var that = this;
		this.map.events.register("zoomend", this, function (e)
		{
			if (that.map.getZoom() < that.zoomLimit) // JSTC
			{
				that.map.setCenter(new OpenLayers.LonLat(0,0),that.zoomLimit,false,true); // JSTC
			}
		});
	},

	initInfoPanel: function()
	{
		// Add legend(s) for layer(s) - NOTE: Called "colors" but applies to color and symbol legends
		$('#colors').show();
		for (var i=0; i<this.layers.length; i++)
		{
			//if (this.types[i] == 'raster')
			var legend = this.wmsServer+"/wms?TRANSPARENT=true&REQUEST=GetLegendGraphic&FORMAT=image/png&LAYER="+this.layers[i];
			if (this.types[i] == 'vector')
			{
				legend += '&STYLE='+this.styles[i];
				$('#colorlegends').append('<div class="colorlabel">&#x25cf;&nbsp;'+this.layers[i]+'</div><img class="symbollegend" src="'+legend+'" alt="symbol legend">');
			}
			else
			{
				$('#colorlegends').append('<div class="colorlabel">&#x25cf;&nbsp;'+this.layers[i]+'</div><img class="colorlegend" src="'+legend+'" alt="color legend">');
			}

		}

		// Fill info pane...
		this.setInfo('<br><br><i>Click on map to get feature info for location.   </i>');

		// Show user which layers cannot be displayed on map
		for (var i=0; i<this.erred.length; i++)
		{
			if (i == 0) // Only show explanatory text if there are some errors
			{
				if (0 == this.layers.length) // Can't display ALL layers
				{
					this.addInfo('<br><br><strong>ERROR: Unable to display layers:</strong><br>');
				}
				else // Only some layers can't be displayed
				{
					this.addInfo('<br><br><strong>NOTE: Unable to overlay the following layers. Try previewing one at a time.</strong><br>');
				}
			}
			this.addInfo('&bull;&nbsp;'+this.erred[i]+'<br>');
		}

		// Make OL items collapsable
		var that = this;
		$('.baseLbl').append('<span class="base-sign sign">'+HIDE+'</span>'); // - Hide -
		$('.baseLbl').click(function(){openClose('base');});
		$('.baseLayersDiv').addClass('base-area');
		$('.dataLbl').not('.copycat').append('<span class="data-sign sign">'+HIDE+'</span>');
		$('.dataLbl').not('.copycat').click(function(){openClose('data');});
		$('.dataLayersDiv').addClass('data-area');

		// Collapse some section by default
		if (!isClosed('basepros'))
		{
			openClose('basepros');
		}
		if (!isClosed('colors'))
		{
			openClose('colors');
		}
	},

	changeBaseProvderTo: function(newProviderId)
	{
		this.baseProvider = newProviderId;  // Set the global

		// Reset then rebuild map using new provider
		this.map.destroy();
		$('#layerswitcher').html('');
		$('#colorlegends' ).html('');
		$('#info'         ).html('');
		this.buildMapStd();
	},

	panelAdjustment: function()
	{
		$('.olControlMousePosition').html('(resizing map)');

		// Adjust Width of map
		$('#map').width( $("#map-viewer").width()  - $("#panel").width() - 12);

		// Redraw map (error occurs if done too frequently)
		if (!this.panelAdjusting)
		{
			var that = this;
			this.panelAdjusting = true;
			setTimeout( function()
				{
					that.panelAdjusting = false; // JSTC
					that.map.updateSize();           // JSTC
				}
				,2000
			);
		}
	},

	// Event callback for WMSGetFeatureInfo - hilights feature, displays its info
	showFeatures: function(evt)
	{
		// Reset info pane and hilight layer
		this.setInfo('<div class="dataLbl">Selected Features</div>');
		this.highlights.destroyFeatures();

		// Any features in response?
		if (evt.features && evt.features.length)
		{
			// Examine each feature
			for (var fi=0; fi<evt.features.length; fi++)
			{
				// Find layer that corresponds to given feature
				var li;
				for (li=0; li<this.layers.length; li++)
				{
					if (evt.features[fi].gml.featureType == this.layers[li])  // Best way to find layer (and proj?)?
					{
						// Re-project feature to map's projection, if possible
						if (evt.features[fi].geometry != null)
						{
							var source = new OpenLayers.Projection(this.projs[li]);  // Use corresponding proj
							var dest   = new OpenLayers.Projection(this.baseProjStr);
							evt.features[fi].geometry.transform(source,dest);
						}

						// Extract and display feature info
						var html = JSON.stringify(evt.features[fi].data);
						html = html.replace(/[{}"]/gm,'')
						html = html.replace(/[:]/gm,'</td><td class="feat_prop">')
						html = '<tr class="feat_prop"><td class="feat_prop">' + html.replace(/[,]/gm,'</td></tr><tr class="feat_prop"><td class="feat_prop">') + '</td></tr>';
						html = '<ul class="featureList"><li>' + '<strong>' + (evt.features[fi].fid || evt.features[fi].gml.featureType)+ '</strong><table class="feat_prop">' + html + '</table></li></ul>';
						this.addInfo(html);

						break;
					}
				}
				if (li >= this.layers.length)
				{
					this.error('Unable to find corresponding layer for selected feature.');
				}
			}

			// Highlight selected feature(s)
			this.highlights.addFeatures(evt.features);
			this.highlights.redraw();
		}
		else
		{
			this.addInfo('<br><i>&nbsp;&nbsp;No features at location.</i>');
		}
	},

	// Setup WMSGetFeatureInfo control so user's click gets feature from WMS
	prepGetFeature: function()
	{
		var that = this;
		var click = new OpenLayers.Control.WMSGetFeatureInfo(
			{
				url          : that.wmsServer + '/' + that.ws + '/wms?' // JSTC
				,title       : 'Identify features by clicking'
				,layers      : that.overlays // JSTC
				,queryVisible: true
				,drillDown   : true
				,infoFormat  : 'application/vnd.ogc.gml'
			}
		);
		click.events.register("getfeatureinfo", this, that.showFeatures);
		this.map.addControl(click);
		click.activate();
	},

	limitBounds: function(projObj,left,bottom,right,top)
	{
		if ('degrees' == projObj.getUnits() || 'EPSG:4326' == projObj.projCode) {
			left   = ((left   <= -180.0) ? -179.99 : left  );
			left   = ((left   >=  180.0) ?  179.99 : left  );
			right  = ((right  <= -180.0) ? -179.99 : right );
			right  = ((right  >=  180.0) ?  179.99 : right );
			bottom = ((bottom <=  -90.0) ?  -89.99 : bottom);
			bottom = ((bottom >=   90.0) ?   89.99 : bottom);
			top    = ((top    <=  -90.0) ?  -89.99 : top   );
			top    = ((top    >=   90.0) ?   89.99 : top   );
		}

		return [left,bottom,right,top];
	},

	// Setup a vector layer to higlight feature(s) selected by user
	createFeatureLayer: function()
	{
		try
		{
			var thisBaseProjStr = this.baseProjStr;
			var highlightStyle  = { fillColor:'#99CCFF', strokeColor:'#3399FF', fillOpacity:0.7 };
			this.highlights = new OpenLayers.Layer.Vector(" Selected Feature(s)",{
							isBaseLayer : false
							,projection : new OpenLayers.Projection(thisBaseProjStr)
							,features   : []
							,visibility : true
							,style      : highlightStyle
						}
					);
			this.map.addLayer(this.highlights);
		}
		catch (e)
		{
			//trace('createFeatureLayer: '+e);
			this.error('Unable to create layer for selected feature(s).');
		}
		//trace('createFeatureLayer');
	},

	addControls: function()
	{
		// Add custom controls to map
		var that = this;
		this.map.addControl(new OpenLayers.Control.LayerSwitcher(
			{
				'div': OpenLayers.Util.getElement('layerswitcher')
			}
		));
		this.map.addControl(new OpenLayers.Control.Navigation());
		this.map.addControl(new OpenLayers.Control.ZoomPanel());
		this.map.addControl(new OpenLayers.Control.ScaleLine());
		this.map.addControl(new OpenLayers.Control.Attribution());
		this.map.addControl(new OpenLayers.Control.MousePosition(
			{
				prefix            : '' // 'LAT/LONG: '
				,displayProjection: new OpenLayers.Projection(that.baseProjStr) // JSTC
				,emptyString      : '(off map)'
				,numDigits        : 3
			}
		));

		$('.olControlZoomInItemInactive'         ).html('+');
		$('.olControlZoomToMaxExtentItemInactive').html('O');
		$('.olControlZoomOutItemInactive'        ).html('&ndash;');
	},

	// Clear and set info window
	setInfo: function(html)
	{
		$('#info').html(html);
	},

	// Append to info window
	addInfo: function(html)
	{
		$('#info').append(html);
	},

	// Format error msg & append to info window
	error: function(msg,e)
	{
		this.setInfo('<br><strong>ERROR: '+msg+'</strong>');
	},
};

// Expand or collapse a panel section

var HIDE = '- Hide -';
var SHOW = '- Show -';

function isClosed(section)
{
	return ($('.'+section+'-sign').html() == SHOW);
}

function openClose(section)
{
	$('.'+section+'-area').slideToggle(100);

	if (this.isClosed(section))
	{
		$('.'+section+'-sign').html(HIDE);
	}
	else {
		$('.'+section+'-sign').html(SHOW);
	}
}

//function trace(msg)
//{
//	console.log(Math.round(performance.now()),msg);
//}
