<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-irods
 * @author    Rob Campbell <rcampbel@purdue.edu>
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// No direct access
defined('_HZEXEC_') or die();

require_once PATH_APP . DS . 'plugins' . DS . 'filesystem' . DS . 'irods' . DS . 'prods.php';

/**
 * Plugin class for irods-specific file handling
 */
class plgHandlersIrods extends \Hubzero\Plugin\Plugin
{
	/**
	 * Determines if the given collection can be handled by this plugin
	 *
	 * @param   \Hubzero\Filesystem\Collection  $collection  The file collection to assess
	 * @return  void
	 **/
	public function canHandle(\Hubzero\Filesystem\Collection $collection)
	{
		// TODO Figure out why previewing non-prj/dbf works? We're returning 'false' for them!

		// Check extension to make sure we can proceed
		if ($collection->hasExtensions(['prj' => 1]) ||
			$collection->hasExtensions(['dbf' => 1])    )
		{
			return true;
		}

		return false;
	}

	/**
	 * Handles view events for irods-specific files
	 *
	 * @param   \Hubzero\Filesystem\Collection $collection  The file collection to view
	 * @return                                 $view        The newly created view
	 **/
	public function onHandleView(\Hubzero\Filesystem\Collection $collection)
	{
		// Note: removed for now: 'asc' => 'map',
		$viewers = [
			'tif'  => 'map'
			,'shp' => 'map'
			,'kml' => 'map'
			,'gml' => 'map'
			,'nc'  => 'map'
			,'dbf' => 'dbf'
			,'prj' => 'prj'
			,'txt' => 'txt'
			,'gif' => 'gif'
			,'jpg' => 'gif'
			,'png' => 'gif'
			,'zip' => 'zip'
			,'gz'  => 'zip'
			,'csv' => 'csv'
			,'tsv' => 'csv'
			,'tab' => 'csv'
		];

		// Find and build correct view
		$lowers = array_change_key_case($collection->getFlatListOfExtensions(), CASE_LOWER);
		foreach ($viewers as $ext => $layout)
		{
			// if ($collection->hasExtensions([$ext => 1]))
			if (array_key_exists($ext, $lowers))
			{
				// Create view
				$view = new \Hubzero\Plugin\View([
					'folder'  => 'handlers',
					'element' => 'irods',
					'name'    => 'irods',
					'layout'  => $layout
				]);

				if ($layout === 'map') // Map preview can handle multiple selections (files)
				{
					$view = $this->MapView($view, $collection);
				}
				else // Other preivews can only handle one file - code will pick one if mult selections
				{
					$view->ext   = $ext;
					$view->limit = $this->params->get('linelimit');

					// Find first file with similar extension (regardless of up/low case)
					foreach ($collection->getFlatListOfFiles() as $file)
					{
						if (strtolower($file->getExtension()) == $ext)
						{
							$view->file = $file;
							break;
						}
					}
				}

				return $view;
			}
		}

		return false;
	}

	/**
	 * Check if given file extension is a supported vector type
	 *
	 * @param   String $ext	file extension
	 * @return  bool   		supported?
	 **/
	protected function supportedVectorExt($ext)
	{
		if ((strtoupper($ext) == 'SHP') ||
			(strtoupper($ext) == 'KML') ||
			(strtoupper($ext) == 'GML')    )
		{
			return true;
		}

		return false;
	}

	/**
	 * Creates map view for selected layer file(s)
	 *
	 * @param   \Hubzero\Filesystem\Collection $collection  The file collection to view
	 * @param   \Hubzero\Plugin\View           $view        A newly created view
	 * @return                                 $view        View populated with map-specific fields/values
	 **/
	protected function MapView(\Hubzero\Plugin\View $view, \Hubzero\Filesystem\Collection $collection)
	{
		$gsurl = $this->params->get('geoserverurl');

		// Replace "localhost" w/local IP in Geoserver URL (for evaluation VM)
		$parts = parse_url($gsurl);
		if (strtoupper($parts['host']) == 'LOCALHOST')
		{
			$gsurl = $parts['scheme'].'://'.getHostByName(getHostName()).':'.$parts['port'].$parts['path'];
		}

		// Init map related view parameters
		$view->gsurl  = $gsurl;
		$view->ws     = '?'; // Ex:'dGVzdGMyL3Rlc3Qx'
		$view->projs  = [];  // Ex: ['4326','4326'];
		$view->layers = [];  // Ex: ['canaryislands_tmo_2013166_geo_Rob_Test_DE_1518_b1','recintos_autonomicas_inspire_canarias_wgs84_rob_test_de_1518'];
		$view->types  = [];  // Ex: ['raster','vector'];
		$view->erred  = [];  // Name of layer/file that is not previewable
		$view->nlim   = [];  // Bounds ...
		$view->elim   = [];
		$view->wlim   = [];
		$view->slim   = [];
		$view->styles = $this->params->get('styles');

		// Get access to iRODS
		$config  = Plugin::params('filesystem', 'irods');
		$account = new RODSAccount(
			$config->get('host')
			, intval($config->get('port'))
			, $config->get('user')
			, $config->get('pass')
			, $config->get('zone')
			, $config->get('resc')
		);

		// Get info for each selected file, add to view
		for ($collection->rewind(), $i=0; $collection->valid(); $collection->next(),$i++)
		{
			$file = $collection->current();
			$base = $file->getAdapter()->getAdapter()->getBaseDir();

			// Prepare to call the map preview microservice
			$inp  = array();
			$inp['*path'] = DS . $config->get('zone') . DS . 'home' . DS . $config->get('user') . DS . $base . $file->path; // NOTE: Hardcoded 'home'

			if ($this->supportedVectorExt($file->getExtension()))
			{
				$body = "previewVector{msiGeoVectorPreview(*path,*layername,*workspace,*projection,*status);}";
			}
			else
			{
				$body = "previewRaster{msiGeoRasterPreview(*path,*band,*layername,*workspace,*projection,*status);}";
				$inp['*band'] = '1';  // NOTE: Hardcoded band: #1
			}

			// Make the microservice call
			$error = false;
			$res   = [];
			for ($retry=0, $limit=3; $retry<$limit; $retry++)
			{
				try
				{
					$out = array("*layername","*workspace","*projection","*status");
					$rule = new ProdsRule($account,$body,$inp,$out);
					$res = $rule->execute();
					if (0 == intval($res["*status"]))
					{
						$error = false;
						break; // Success!
					}
				}
				catch (Exception $e)
				{
					error_log('Map preview microservice for "'.$file->getName().'": '.$e);
					if ($limit == 1+$retry)
					{
						// Error, record bad layer name, move on
						$view->erred[] = $file->getName();
						$error = true;
					}
				}
			}

			if ($error)
			{
				continue;
			}

			// No error - Add view paramters for this layer
			$view->ws       = $res["*workspace"];  // NOTE: Always using most recent. Assume all layers in same workspace
			$coll_layers    = explode(",",$res["*layername"]);

			foreach($coll_layers as $layer)
			{
				$view->layers[] = $layer;
				$view->projs[]  = $res["*projection"];

				if ($this->supportedVectorExt($file->getExtension()))
				{
					$view->types[] = 'vector';
				}
				else
				{
					$view->types[] = 'raster';
				}
			}

			// Get bounds from metadata

			$view->nlim[$i] = '0'; 		// Set default values
			$view->elim[$i] = '0';
			$view->wlim[$i] = '0';
			$view->slim[$i] = '0';

			$prodsf = Prods::getProdsFile($file);	// Get metadata
			$prodsm = $prodsf->getMeta();

			foreach ($prodsm as $rodsmeta)	// Extract bounds
			{
				// TODO Add sanity checks
				if($res["*projection"] == '4326')  // Stand GABBS projection
				{
					switch ($rodsmeta->name)
					{
						case 'latmax': $view->nlim[$i] = $rodsmeta->value; break;
						case 'lonmax': $view->elim[$i] = $rodsmeta->value; break;
						case 'lonmin': $view->wlim[$i] = $rodsmeta->value; break;
						case 'latmin': $view->slim[$i] = $rodsmeta->value; break;
					}
				}
				else
				{
					switch ($rodsmeta->name)
					{
						case 'northlimit': $view->nlim[$i] = $rodsmeta->value; break;
						case 'eastlimit' : $view->elim[$i] = $rodsmeta->value; break;
						case 'westlimit' : $view->wlim[$i] = $rodsmeta->value; break;
						case 'southlimit': $view->slim[$i] = $rodsmeta->value; break;
					}
				}
			}
		}

		return $view;
	}
}
