<?php
/**
 * HUBzero CMS
 *
 * Copyright 2005-2015 HUBzero Foundation, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * HUBzero is a registered trademark of Purdue University.
 *
 * @package   hubzero-irods
 * @author    Rob Campbell <rcampbel@purdue.edu>
 * @copyright Copyright 2005-2015 HUBzero Foundation, LLC.
 * @license   http://opensource.org/licenses/MIT MIT
 */

// No direct access
defined('_HZEXEC_') or die();

require_once PATH_APP . DS . 'plugins' . DS . 'filesystem' . DS . 'irods' . DS . 'prods.php';

/**
 * Plugin class for iRODS metadata handling
 */
class plgMetadataIrods extends \Hubzero\Plugin\Plugin
{
	/**
	* Responds to events for rendering the metadata edit interface
	*
	* @return  void
	**/
	public function onMetadataEdit()
	{
		$view = new \Hubzero\Plugin\View([
			'folder'  => 'metadata',
			'element' => 'irods',
			'name'    => 'irods',
			'layout'  => 'edit'
		]);

		$hidden = array();

		//group is no longer used, superseded by owner and owner_type
		//we include group here since it may be present in older files
		$hidden[] = 'group';
		$hidden[] = 'owner';
		$hidden[] = 'owner_type';
		$view->hideFields = $hidden;

		return $view;
	}

	/**
	* Responds to events for saving file metadata
	*
	* @param   Hubzero\Filesystem\File  $file      The file to which the metadata pertains
	* @param   array                    $metadata  The metadata itself
	* @return  void
	**/
	public function onMetadataSave(Hubzero\Filesystem\File $file, $metadata)
	{
		// NOTE Only those key/value pairs that have NON-EMPTY values
		//      are being passed to us in $metadata!

		// Get key/value pairs for exisiting data
		$prodsf = Prods::getProdsFile($file);
		$old    = $prodsf->getMeta();
		$old_kv = array();

		foreach ($old as $rodsmeta)
		{
			if ( '' == trim($rodsmeta->units))
			{
				$old_kv[$rodsmeta->name] = $rodsmeta->value;
			}
			else
			{
				$old_kv[$rodsmeta->name . ':' . $rodsmeta->units] = $rodsmeta->value;
			}
		}

		// Create union of all keys
		$master = array_keys(($old_kv + $metadata), $strict=true);

		// Build list of updates
		$update = array();
		foreach ($master as $key)
		{
			if (($key === 'group') ||
			    ($key === 'owner') ||
			    ($key === 'owner_type'))
			{
				continue;
			}

			if (array_key_exists($key, $old_kv))
			{
				if (array_key_exists($key, $metadata))
				{
					if (!($old_kv[$key] === $metadata[$key]))
					{
						$update[$key] = $metadata[$key];  // Value differs
					}
				}
				else
				{
					$update[$key] = 'null'; // Key in old but not new. "null" = signal to microservice to delete
				}
			}
			else
			{
				$update[$key] = $metadata[$key]; // Key in new but not old
			}
		}

		$res = plgMetadataIrods::bulkMeta($prodsf,$update);
	}

	/**
	* Responds to events for getting the latest metadata annotation set
	*
	* @param   Hubzero\Filesystem\File  $file        The file to which the metadata pertains
	* @param   int                      $maxEntries  The maximum number of entries to return
	* @return  array
	**/
	public function onMetadataGet(Hubzero\Filesystem\File $file, $maxEntries = 1)
	{
		$prodsf = Prods::getProdsFile($file);
		$prodsm = $prodsf->getMeta();

		$metadata = [];

		// Append key with ":<units>"
		foreach ($prodsm as $rodsmeta)
		{
			$metadata[$rodsmeta->name] = ['value' => $rodsmeta->value, 'units' => $rodsmeta->units];
		}

		return $metadata;
	}

	/**
	* Write metadata for a prods file
	*
	* @param   string  object     Prods file
	* @param   array   metadata   key/value pairs
	* @return  void
	**/
	private static function bulkMeta($prodsf, $metadata)
	{
		// Build the iRODS function call
		$kvpairs = new RODSKeyValPair($metadata);
		$body    = "updateBulkMeta{msiSetBulkMeta(*KeyVal,*path,*status);}";
		$inp     = array();
		$inp['*KeyVal'] = $kvpairs;
		$inp['*path']   = $prodsf->path_str;
		$out     = array("*status");

		// Get access
		$config  = Plugin::params('filesystem', 'irods');
		$account = new RODSAccount(
			$config->get('host')
			, intval($config->get('port'))
			, $config->get('user')
			, $config->get('pass')
			, $config->get('zone')
			, $config->get('resc')
		);

		// Perform the iRODS funtion call
		$rule = new ProdsRule($account,$body,$inp,$out);
		$res = $rule->execute();
		return($res["*status"]);
	}

	/**
	* Derive and record metadata for new file
	*
	* @param   string  $path        Full path to new file
	* @return  void
	**/
	public static function captureInitialMetadata($path)
	{
		$prodsf = Prods::prodsFileConnect($path);
		$config = Plugin::params('metadata', 'irods');

		$metadata = array();
		$metadata['contributor'] = User::get('username');
		$metadata['publisher']   = $config->get('publisher');

		plgMetadataIrods::bulkMeta($prodsf,$metadata);
	}
}
